@@ -11,13 +11,13 @@ import re
 from .. import utils
 from .decorator import BaseDecorator
 
-logger = logging.getLogger("firetail.decorators.uri_parsing")
+logger = logging.getLogger('firetail.decorators.uri_parsing')
 
 QUERY_STRING_DELIMITERS = {
-    "spaceDelimited": " ",
-    "pipeDelimited": "|",
-    "simple": ",",
-    "form": ",",
+    'spaceDelimited': ' ',
+    'pipeDelimited': '|',
+    'simple': ',',
+    'form': ','
 }
 
 
@@ -35,9 +35,9 @@ class AbstractURIParser(BaseDecorator, metaclass=abc.ABCMeta):
         - https://mysite.fake/?in_query=a|b|c           # various separators
         - https://mysite.fake/?in_query=a&in_query=b,c  # complex query params
         """
-        self._param_defns = {
-            p["name"]: p for p in param_defns if p["in"] in self.parsable_parameters
-        }
+        self._param_defns = {p["name"]: p
+                             for p in param_defns
+                             if p["in"] in self.parsable_parameters}
         self._body_schema = body_defn.get("schema", {})
         self._body_encoding = body_defn.get("encoding", {})
 
@@ -60,27 +60,29 @@ class AbstractURIParser(BaseDecorator, metaclass=abc.ABCMeta):
         :rtype: str
         """
         return "<{classname}>".format(
-            classname=self.__class__.__name__
-        )  # pragma: no cover
+            classname=self.__class__.__name__)  # pragma: no cover
 
     @abc.abstractmethod
     def resolve_form(self, form_data):
-        """Resolve cases where form parameters are provided multiple times."""
+        """ Resolve cases where form parameters are provided multiple times.
+        """
 
     @abc.abstractmethod
     def resolve_query(self, query_data):
-        """Resolve cases where query parameters are provided multiple times."""
+        """ Resolve cases where query parameters are provided multiple times.
+        """
 
     @abc.abstractmethod
     def resolve_path(self, path):
-        """Resolve cases where path parameters include lists"""
+        """ Resolve cases where path parameters include lists
+        """
 
     @abc.abstractmethod
     def _resolve_param_duplicates(self, values, param_defn, _in):
-        """Resolve cases where query parameters are provided multiple times.
-        For example, if the query string is '?a=1,2,3&a=4,5,6' the value of
-        `a` could be "4,5,6", or "1,2,3" or "1,2,3,4,5,6" depending on the
-        implementation.
+        """ Resolve cases where query parameters are provided multiple times.
+            For example, if the query string is '?a=1,2,3&a=4,5,6' the value of
+            `a` could be "4,5,6", or "1,2,3" or "1,2,3,4,5,6" depending on the
+            implementation.
         """
 
     @abc.abstractmethod
@@ -107,11 +109,11 @@ class AbstractURIParser(BaseDecorator, metaclass=abc.ABCMeta):
                 resolved_param[k] = values
                 continue
 
-            if _in == "path":
+            if _in == 'path':
                 # multiple values in a path is impossible
                 values = [values]
 
-            if param_schema and param_schema["type"] == "array":
+            if param_schema and param_schema['type'] == 'array':
                 # resolve variable re-assignment, handle explode
                 values = self._resolve_param_duplicates(values, param_defn, _in)
                 # handle array styles
@@ -130,7 +132,8 @@ class AbstractURIParser(BaseDecorator, metaclass=abc.ABCMeta):
         @functools.wraps(function)
         def wrapper(request):
             def coerce_dict(md):
-                """MultiDict -> dict of lists"""
+                """ MultiDict -> dict of lists
+                """
                 try:
                     return md.to_dict(flat=False)
                 except AttributeError:
@@ -150,13 +153,9 @@ class AbstractURIParser(BaseDecorator, metaclass=abc.ABCMeta):
 
 
 class OpenAPIURIParser(AbstractURIParser):
-    style_defaults = {
-        "path": "simple",
-        "header": "simple",
-        "query": "form",
-        "cookie": "form",
-        "form": "form",
-    }
+    style_defaults = {"path": "simple", "header": "simple",
+                      "query": "form", "cookie": "form",
+                      "form": "form"}
 
     @property
     def param_defns(self):
@@ -164,33 +163,30 @@ class OpenAPIURIParser(AbstractURIParser):
 
     @property
     def form_defns(self):
-        return {k: v for k, v in self._body_schema.get("properties", {}).items()}
+        return {k: v for k, v in self._body_schema.get('properties', {}).items()}
 
     @property
     def param_schemas(self):
-        return {k: v.get("schema", {}) for k, v in self.param_defns.items()}
+        return {k: v.get('schema', {}) for k, v in self.param_defns.items()}
 
     def resolve_form(self, form_data):
-        if self._body_schema is None or self._body_schema.get("type") != "object":
+        if self._body_schema is None or self._body_schema.get('type') != 'object':
             return form_data
         for k in form_data:
             encoding = self._body_encoding.get(k, {"style": "form"})
             defn = self.form_defns.get(k, {})
             # TODO support more form encoding styles
-            form_data[k] = self._resolve_param_duplicates(
-                form_data[k], encoding, "form"
-            )
+            form_data[k] = \
+                self._resolve_param_duplicates(form_data[k], encoding, 'form')
             if defn and defn["type"] == "array":
-                form_data[k] = self._split(form_data[k], encoding, "form")
-            elif "contentType" in encoding and utils.all_json(
-                [encoding.get("contentType")]
-            ):
+                form_data[k] = self._split(form_data[k], encoding, 'form')
+            elif 'contentType' in encoding and utils.all_json([encoding.get('contentType')]):
                 form_data[k] = json.loads(form_data[k])
         return form_data
 
     def _make_deep_object(self, k, v):
-        """consumes keys, value pairs like (a[foo][bar], "baz")
-        returns (a, {"foo": {"bar": "baz"}}}, is_deep_object)
+        """ consumes keys, value pairs like (a[foo][bar], "baz")
+            returns (a, {"foo": {"bar": "baz"}}}, is_deep_object)
         """
         root_key = None
         if k in self.param_schemas.keys():
@@ -198,7 +194,7 @@ class OpenAPIURIParser(AbstractURIParser):
         else:
             for key in self.param_schemas.keys():
                 if k.startswith(key) and "[" in k:
-                    root_key = key.replace(k, "")
+                    root_key = key.replace(k, '')
 
         if not root_key:
             root_key = k.split("[", 1)[0]
@@ -208,7 +204,7 @@ class OpenAPIURIParser(AbstractURIParser):
         if not self._is_deep_object_style_param(root_key):
             return k, v, False
 
-        key_path = re.findall(r"\[([^\[\]]*)\]", k)
+        key_path = re.findall(r'\[([^\[\]]*)\]', k)
         root = prev = node = {}
         for k in key_path:
             node[k] = {}
@@ -219,12 +215,12 @@ class OpenAPIURIParser(AbstractURIParser):
 
     def _is_deep_object_style_param(self, param_name):
         default_style = self.style_defaults["query"]
-        style = self.param_defns.get(param_name, {}).get("style", default_style)
+        style = self.param_defns.get(param_name, {}).get('style', default_style)
         return style == "deepObject"
 
     def _preprocess_deep_objects(self, query_data):
-        """deep objects provide a way of rendering nested objects using query
-        parameters.
+        """ deep objects provide a way of rendering nested objects using query
+            parameters.
         """
         deep = [self._make_deep_object(k, v) for k, v in query_data.items()]
         root_keys = [k for k, v, is_deep_object in deep]
@@ -238,25 +234,25 @@ class OpenAPIURIParser(AbstractURIParser):
 
     def resolve_query(self, query_data):
         query_data = self._preprocess_deep_objects(query_data)
-        return self.resolve_params(query_data, "query")
+        return self.resolve_params(query_data, 'query')
 
     def resolve_path(self, path_data):
-        return self.resolve_params(path_data, "path")
+        return self.resolve_params(path_data, 'path')
 
     @staticmethod
     def _resolve_param_duplicates(values, param_defn, _in):
-        """Resolve cases where query parameters are provided multiple times.
-        The default behavior is to use the first-defined value.
-        For example, if the query string is '?a=1,2,3&a=4,5,6' the value of
-        `a` would be "4,5,6".
-        However, if 'explode' is 'True' then the duplicate values
-        are concatenated together and `a` would be "1,2,3,4,5,6".
+        """ Resolve cases where query parameters are provided multiple times.
+            The default behavior is to use the first-defined value.
+            For example, if the query string is '?a=1,2,3&a=4,5,6' the value of
+            `a` would be "4,5,6".
+            However, if 'explode' is 'True' then the duplicate values
+            are concatenated together and `a` would be "1,2,3,4,5,6".
         """
         default_style = OpenAPIURIParser.style_defaults[_in]
-        style = param_defn.get("style", default_style)
-        delimiter = QUERY_STRING_DELIMITERS.get(style, ",")
-        is_form = style == "form"
-        explode = param_defn.get("explode", is_form)
+        style = param_defn.get('style', default_style)
+        delimiter = QUERY_STRING_DELIMITERS.get(style, ',')
+        is_form = (style == 'form')
+        explode = param_defn.get('explode', is_form)
         if explode:
             return delimiter.join(values)
 
@@ -266,8 +262,8 @@ class OpenAPIURIParser(AbstractURIParser):
     @staticmethod
     def _split(value, param_defn, _in):
         default_style = OpenAPIURIParser.style_defaults[_in]
-        style = param_defn.get("style", default_style)
-        delimiter = QUERY_STRING_DELIMITERS.get(style, ",")
+        style = param_defn.get('style', default_style)
+        delimiter = QUERY_STRING_DELIMITERS.get(style, ',')
         return value.split(delimiter)
 
 
@@ -276,7 +272,6 @@ class Swagger2URIParser(AbstractURIParser):
     Adheres to the Swagger2 spec,
     Assumes the the last defined query parameter should be used.
     """
-
     parsable_parameters = ["query", "path", "formData"]
 
     @property
@@ -288,33 +283,33 @@ class Swagger2URIParser(AbstractURIParser):
         return self._param_defns  # swagger2 conflates defn and schema
 
     def resolve_form(self, form_data):
-        return self.resolve_params(form_data, "form")
+        return self.resolve_params(form_data, 'form')
 
     def resolve_query(self, query_data):
-        return self.resolve_params(query_data, "query")
+        return self.resolve_params(query_data, 'query')
 
     def resolve_path(self, path_data):
-        return self.resolve_params(path_data, "path")
+        return self.resolve_params(path_data, 'path')
 
     @staticmethod
     def _resolve_param_duplicates(values, param_defn, _in):
-        """Resolve cases where query parameters are provided multiple times.
-        The default behavior is to use the first-defined value.
-        For example, if the query string is '?a=1,2,3&a=4,5,6' the value of
-        `a` would be "4,5,6".
-        However, if 'collectionFormat' is 'multi' then the duplicate values
-        are concatenated together and `a` would be "1,2,3,4,5,6".
+        """ Resolve cases where query parameters are provided multiple times.
+            The default behavior is to use the first-defined value.
+            For example, if the query string is '?a=1,2,3&a=4,5,6' the value of
+            `a` would be "4,5,6".
+            However, if 'collectionFormat' is 'multi' then the duplicate values
+            are concatenated together and `a` would be "1,2,3,4,5,6".
         """
-        if param_defn.get("collectionFormat") == "multi":
-            return ",".join(values)
+        if param_defn.get('collectionFormat') == 'multi':
+            return ','.join(values)
         # default to last defined value
         return values[-1]
 
     @staticmethod
     def _split(value, param_defn, _in):
-        if param_defn.get("collectionFormat") == "pipes":
-            return value.split("|")
-        return value.split(",")
+        if param_defn.get("collectionFormat") == 'pipes':
+            return value.split('|')
+        return value.split(',')
 
 
 class FirstValueURIParser(Swagger2URIParser):
@@ -325,15 +320,15 @@ class FirstValueURIParser(Swagger2URIParser):
 
     @staticmethod
     def _resolve_param_duplicates(values, param_defn, _in):
-        """Resolve cases where query parameters are provided multiple times.
-        The default behavior is to use the first-defined value.
-        For example, if the query string is '?a=1,2,3&a=4,5,6' the value of
-        `a` would be "1,2,3".
-        However, if 'collectionFormat' is 'multi' then the duplicate values
-        are concatenated together and `a` would be "1,2,3,4,5,6".
+        """ Resolve cases where query parameters are provided multiple times.
+            The default behavior is to use the first-defined value.
+            For example, if the query string is '?a=1,2,3&a=4,5,6' the value of
+            `a` would be "1,2,3".
+            However, if 'collectionFormat' is 'multi' then the duplicate values
+            are concatenated together and `a` would be "1,2,3,4,5,6".
         """
-        if param_defn.get("collectionFormat") == "multi":
-            return ",".join(values)
+        if param_defn.get('collectionFormat') == 'multi':
+            return ','.join(values)
         # default to first defined value
         return values[0]
 
@@ -346,11 +341,11 @@ class AlwaysMultiURIParser(Swagger2URIParser):
 
     @staticmethod
     def _resolve_param_duplicates(values, param_defn, _in):
-        """Resolve cases where query parameters are provided multiple times.
-        The default behavior is to join all provided parameters together.
-        For example, if the query string is '?a=1,2,3&a=4,5,6' the value of
-        `a` would be "1,2,3,4,5,6".
+        """ Resolve cases where query parameters are provided multiple times.
+            The default behavior is to join all provided parameters together.
+            For example, if the query string is '?a=1,2,3&a=4,5,6' the value of
+            `a` would be "1,2,3,4,5,6".
         """
-        if param_defn.get("collectionFormat") == "pipes":
-            return "|".join(values)
-        return ",".join(values)
+        if param_defn.get('collectionFormat') == 'pipes':
+            return '|'.join(values)
+        return ','.join(values)
