@@ -3,6 +3,7 @@ This module defines an OpenAPIOperation class, a Firetail operation specific for
 """
 
 import logging
+import warnings
 from copy import copy, deepcopy
 
 from firetail.operations.abstract import AbstractOperation
@@ -20,24 +21,10 @@ class OpenAPIOperation(AbstractOperation):
     A single API operation on a path.
     """
 
-    def __init__(
-        self,
-        api,
-        method,
-        path,
-        operation,
-        resolver,
-        path_parameters=None,
-        app_security=None,
-        security_schemes=None,
-        components=None,
-        validate_responses=False,
-        strict_validation=False,
-        randomize_endpoint=None,
-        validator_map=None,
-        pythonic_params=False,
-        uri_parser_class=None,
-    ):
+    def __init__(self, api, method, path, operation, resolver, path_parameters=None,
+                 app_security=None, components=None, validate_responses=False,
+                 strict_validation=False, randomize_endpoint=None, validator_map=None,
+                 pythonic_params=False, uri_parser_class=None, pass_context_arg_name=None):
         """
         This class uses the OperationID identify the module and function that will handle the operation
 
@@ -59,9 +46,6 @@ class OpenAPIOperation(AbstractOperation):
         :type path_parameters: list
         :param app_security: list of security rules the application uses by default
         :type app_security: list
-        :param security_schemes: `Security Definitions Object
-            <https://github.com/swagger-api/swagger-spec/blob/master/versions/2.0.md#security-definitions-object>`_
-        :type security_schemes: dict
         :param components: `Components Object
             <https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.1.md#componentsObject>`_
         :type components: dict
@@ -78,12 +62,21 @@ class OpenAPIOperation(AbstractOperation):
         :type pythonic_params: bool
         :param uri_parser_class: class to use for uri parsing
         :type uri_parser_class: AbstractURIParser
+        :param pass_context_arg_name: If not None will try to inject the request context to the function using this
+            name.
+        :type pass_context_arg_name: str|None
         """
         self.components = components or {}
 
+        def component_get(oas3_name):
+            return self.components.get(oas3_name, {})
+
+        # operation overrides globals
+        security_schemes = component_get('securitySchemes')
+        app_security = operation.get('security', app_security)
         uri_parser_class = uri_parser_class or OpenAPIURIParser
 
-        self._router_controller = operation.get("x-openapi-router-controller")
+        self._router_controller = operation.get('x-openapi-router-controller')
 
         super().__init__(
             api=api,
@@ -99,29 +92,42 @@ class OpenAPIOperation(AbstractOperation):
             validator_map=validator_map,
             pythonic_params=pythonic_params,
             uri_parser_class=uri_parser_class,
+            pass_context_arg_name=pass_context_arg_name
         )
 
-        self._request_body = operation.get("requestBody", {})
+        self._definitions_map = {
+            'components': {
+                'schemas': component_get('schemas'),
+                'examples': component_get('examples'),
+                'requestBodies': component_get('requestBodies'),
+                'parameters': component_get('parameters'),
+                'securitySchemes': component_get('securitySchemes'),
+                'responses': component_get('responses'),
+                'headers': component_get('headers'),
+            }
+        }
 
-        self._parameters = operation.get("parameters", [])
+        self._request_body = operation.get('requestBody', {})
+
+        self._parameters = operation.get('parameters', [])
         if path_parameters:
             self._parameters += path_parameters
 
-        self._responses = operation.get("responses", {})
+        self._responses = operation.get('responses', {})
 
         # TODO figure out how to support multiple mimetypes
         # NOTE we currently just combine all of the possible mimetypes,
         #      but we need to refactor to support mimetypes by response code
         response_content_types = []
         for _, defn in self._responses.items():
-            response_content_types += defn.get("content", {}).keys()
-        self._produces = response_content_types or ["application/json"]
+            response_content_types += defn.get('content', {}).keys()
+        self._produces = response_content_types or ['application/json']
 
-        request_content = self._request_body.get("content", {})
-        self._consumes = list(request_content.keys()) or ["application/json"]
+        request_content = self._request_body.get('content', {})
+        self._consumes = list(request_content.keys()) or ['application/json']
 
-        logger.debug("consumes: %s" % self.consumes)
-        logger.debug("produces: %s" % self.produces)
+        logger.debug('consumes: %s' % self.consumes)
+        logger.debug('produces: %s' % self.produces)
 
     @classmethod
     def from_spec(cls, spec, api, path, method, resolver, *args, **kwargs):
@@ -133,10 +139,9 @@ class OpenAPIOperation(AbstractOperation):
             resolver=resolver,
             path_parameters=spec.get_path_params(path),
             app_security=spec.security,
-            security_schemes=spec.security_schemes,
             components=spec.components,
             *args,
-            **kwargs,
+            **kwargs
         )
 
     @property
@@ -157,11 +162,13 @@ class OpenAPIOperation(AbstractOperation):
 
     def with_definitions(self, schema):
         if self.components:
-            schema["schema"]["components"] = self.components
+            schema['schema']['components'] = self.components
         return schema
 
     def response_schema(self, status_code=None, content_type=None):
-        response_definition = self.response_definition(status_code, content_type)
+        response_definition = self.response_definition(
+            status_code, content_type
+        )
         content_definition = response_definition.get("content", response_definition)
         content_definition = content_definition.get(content_type, content_definition)
         if "schema" in content_definition:
@@ -176,16 +183,12 @@ class OpenAPIOperation(AbstractOperation):
         status_code = status_code or sorted(self._responses.keys())[0]
 
         content_type = content_type or self.get_mimetype()
-        examples_path = [str(status_code), "content", content_type, "examples"]
-        example_path = [str(status_code), "content", content_type, "example"]
+        examples_path = [str(status_code), 'content', content_type, 'examples']
+        example_path = [str(status_code), 'content', content_type, 'example']
         schema_example_path = [
-            str(status_code),
-            "content",
-            content_type,
-            "schema",
-            "example",
+            str(status_code), 'content', content_type, 'schema', 'example'
         ]
-        schema_path = [str(status_code), "content", content_type, "schema"]
+        schema_path = [str(status_code), 'content', content_type, 'schema']
 
         try:
             status_code = int(status_code)
@@ -194,8 +197,8 @@ class OpenAPIOperation(AbstractOperation):
         try:
             # TODO also use example header?
             return (
-                list(deep_get(self._responses, examples_path).values())[0]["value"],
-                status_code,
+                list(deep_get(self._responses, examples_path).values())[0]['value'],
+                status_code
             )
         except (KeyError, IndexError):
             pass
@@ -204,15 +207,14 @@ class OpenAPIOperation(AbstractOperation):
         except KeyError:
             pass
         try:
-            return (deep_get(self._responses, schema_example_path), status_code)
+            return (deep_get(self._responses, schema_example_path),
+                    status_code)
         except KeyError:
             pass
 
         try:
-            return (
-                self._nested_example(deep_get(self._responses, schema_path)),
-                status_code,
-            )
+            return (self._nested_example(deep_get(self._responses, schema_path)),
+                    status_code)
         except KeyError:
             return (None, status_code)
 
@@ -223,10 +225,8 @@ class OpenAPIOperation(AbstractOperation):
             pass
         try:
             # Recurse if schema is an object
-            return {
-                key: self._nested_example(value)
-                for (key, value) in schema["properties"].items()
-            }
+            return {key: self._nested_example(value)
+                    for (key, value) in schema["properties"].items()}
         except KeyError:
             pass
         try:
@@ -240,15 +240,12 @@ class OpenAPIOperation(AbstractOperation):
         path_parameters = (p for p in self.parameters if p["in"] == "path")
         for path_defn in path_parameters:
             path_schema = path_defn["schema"]
-            if (
-                path_schema.get("type") == "string"
-                and path_schema.get("format") == "path"
-            ):
+            if path_schema.get('type') == 'string' and path_schema.get('format') == 'path':
                 # path is special case for type 'string'
-                path_type = "path"
+                path_type = 'path'
             else:
-                path_type = path_schema.get("type")
-            types[path_defn["name"]] = path_type
+                path_type = path_schema.get('type')
+            types[path_defn['name']] = path_type
         return types
 
     @property
@@ -256,7 +253,7 @@ class OpenAPIOperation(AbstractOperation):
         """
         The body schema definition for this operation.
         """
-        return self.body_definition.get("schema", {})
+        return self.body_definition.get('schema', {})
 
     @property
     def body_definition(self):
@@ -270,10 +267,9 @@ class OpenAPIOperation(AbstractOperation):
         if self._request_body:
             if len(self.consumes) > 1:
                 logger.warning(
-                    "this operation accepts multiple content types, using %s",
-                    self.consumes[0],
-                )
-            res = self._request_body.get("content", {}).get(self.consumes[0], {})
+                    'this operation accepts multiple content types, using %s',
+                    self.consumes[0])
+            res = self._request_body.get('content', {}).get(self.consumes[0], {})
             return self.with_definitions(res)
         return {}
 
@@ -281,7 +277,16 @@ class OpenAPIOperation(AbstractOperation):
         if len(arguments) <= 0 and not has_kwargs:
             return {}
 
-        x_body_name = sanitize(self.request_body.get("x-body-name", "body"))
+        # get the deprecated name from the body-schema for legacy firetail compat
+        x_body_name = sanitize(self.body_schema.get('x-body-name'))
+
+        if x_body_name:
+            warnings.warn('x-body-name within the requestBody schema will be deprecated in the '
+                          'next major version. It should be provided directly under '
+                          'the requestBody instead.', DeprecationWarning)
+
+        # prefer the x-body-name as an extension of requestBody, fallback to deprecated schema name, default 'body'
+        x_body_name = sanitize(self.request_body.get('x-body-name', x_body_name or 'body'))
 
         if self.consumes[0] in FORM_CONTENT_TYPES:
             result = self._get_body_argument_form(body)
@@ -299,17 +304,16 @@ class OpenAPIOperation(AbstractOperation):
             return None
 
         if body is None:
-            default_body = self.body_schema.get("default", {})
+            default_body = self.body_schema.get('default', {})
             return deepcopy(default_body)
 
         return body
 
     def _get_body_argument_form(self, body):
         # now determine the actual value for the body (whether it came in or is default)
-        default_body = self.body_schema.get("default", {})
-        body_props = {
-            k: {"schema": v} for k, v in self.body_schema.get("properties", {}).items()
-        }
+        default_body = self.body_schema.get('default', {})
+        body_props = {k: {"schema": v} for k, v
+                      in self.body_schema.get("properties", {}).items()}
 
         # by OpenAPI specification `additionalProperties` defaults to `true`
         # see: https://github.com/OAI/OpenAPI-Specification/blame/3.0.2/versions/3.0.2.md#L2305
@@ -333,9 +337,7 @@ class OpenAPIOperation(AbstractOperation):
         :type additional_props: dict|bool
         :rtype: dict
         """
-        additional_props_defn = (
-            {"schema": additional_props} if isinstance(additional_props, dict) else None
-        )
+        additional_props_defn = {"schema": additional_props} if isinstance(additional_props, dict) else None
         res = {}
 
         for key, value in body_arg.items():
@@ -353,15 +355,14 @@ class OpenAPIOperation(AbstractOperation):
         return res
 
     def _build_default_obj_recursive(self, _properties, res):
-        """takes disparate and nested default keys, and builds up a default object"""
+        """ takes disparate and nested default keys, and builds up a default object
+        """
         for key, prop in _properties.items():
-            if "default" in prop and key not in res:
-                res[key] = copy(prop["default"])
-            elif prop.get("type") == "object" and "properties" in prop:
+            if 'default' in prop and key not in res:
+                res[key] = copy(prop['default'])
+            elif prop.get('type') == 'object' and 'properties' in prop:
                 res.setdefault(key, {})
-                res[key] = self._build_default_obj_recursive(
-                    prop["properties"], res[key]
-                )
+                res[key] = self._build_default_obj_recursive(prop['properties'], res[key])
         return res
 
     def _get_default_obj(self, schema):
@@ -384,14 +385,15 @@ class OpenAPIOperation(AbstractOperation):
         return defaults
 
     def _get_query_arguments(self, query, arguments, has_kwargs, sanitize):
-        query_defns = {p["name"]: p for p in self.parameters if p["in"] == "query"}
+        query_defns = {p["name"]: p
+                       for p in self.parameters
+                       if p["in"] == "query"}
         default_query_params = self._get_query_defaults(query_defns)
 
         query_arguments = deepcopy(default_query_params)
         query_arguments = deep_merge(query_arguments, query)
-        return self._query_args_helper(
-            query_defns, query_arguments, arguments, has_kwargs, sanitize
-        )
+        return self._query_args_helper(query_defns, query_arguments,
+                                       arguments, has_kwargs, sanitize)
 
     def _get_val_from_param(self, value, query_defn):
         query_schema = query_defn["schema"]
